% This is a demo script for LCMM analysis

% Set seed for random number generation
rng(0,'twister');

% Simulation setup
nChan   = 10;
nSource = 10; 
sampfreq = 75;%in Hz
am_sampfreq = 1;
nSample   = 150;
nBlock    = 50;
fBand = [ 9.5 10.5 ];
K0  = 3; %true # of classes

% Randomly generate latent classes
y0 = randi(K0, 1, nBlock);
B0 = max(randn(nSource,K0),0);
while any(sum(B0>0.05,1)<=1) %avoid too small variances
    B0 = max(randn(nSource,K0),0);
end
Z0 = sparse(y0, 1:nBlock, 1, K0, nBlock);

% Reordering for the ease of interpretation 
[~,imax]=max(Z0,[],1);
[~,order]=sort(imax,'ascend');
y0 = y0(order);
Z0 = Z0(:,order);
Z0 = repmat(reshape(full(Z0), [K0,1,nBlock]), [1 nSample 1]); %for each sample

% Generate sources
S0 = zeros(nSource,nSample,nBlock);
nMargin = nSample; %margins to avoid boundary artifacts
for i=1:nBlock
    for j=1:nSource
        s = 2.*rand(nSample+2.*nMargin,1)-1; %in [-1,1]
        s = bandpass(s, fBand, sampfreq)';
        S0(j,:,i) = s(1,nMargin+(1:nSample)); %cut the margins
    end
end
S0(:,:) = bsxfun(@minus, S0(:,:), mean(S0(:,:),2)); %centering

% Block-wise rescaling
for i=1:nBlock
    S0(:,:,i) = diag(B0(:,y0(i)))*S0(:,:,i);
end

% Global amplitude modulation
U0 = zeros(nSample,nBlock);
x=0:pi/180:2.*pi; x(end)=[];
for i=1:nBlock
    U0(:,i) = 0.9.*sin(2.*pi.*(am_sampfreq).*((0:nSample-1)./sampfreq) + randi(360)*pi/180) + 1;
    for j=1:nSample
        U0(j,i) = gamrnd(2.*U0(j,i),1/2,1,1); %gamma noise with mean U0
    end
end
S0(:,:) = S0(:,:)*diag(U0(:));
S0(:,:) = S0(:,:) + 0.5.*std(S0(:)).*randn(size(S0(:,:))); %add noise

% Linear mixing into observations
A0 = randn(nChan, nSource);
A0 = A0*diag(1./sqrt(sum(A0.*A0,1))); %unit norm
W0 = pinv(A0);
X0 = zeros(nChan,nSample,nBlock);
X0(:,:) = A0*S0(:,:);


% Display true states and sources
figure;
subplot(3,1,1);
imagesc(Z0(:,:));title('True state transition');
subplot(3,1,2);
plotwaves(S0(:,:),sampfreq);title('True sources');
subplot(3,1,3);
plotwaves(X0(:,:),sampfreq);title('Observed signals');

% Run LCMM in alpha band
figure;
X = hilbertdata(X0, sampfreq, [8 12]); 
K = 3;
[S,Pk,A,W,B,Eta,f] = lcmm(X(:,:),K,'display',1);
