function p=parseargs(p,v)
%PARSEARGS Parse optional args
%  p=parseargs(p,v);
%  Usage:
%    function y=fun(x,varargin)
%      %Set defaults
%      p=struct('name1',val1,'name2',val2...);
%      %Get options from name-value pair arguments
%      p=parseargs(p,varargin);
%
%  09-Jun-2015 JH

%Check if input arguments are correctly given
optnames = fieldnames(p);
names = v(1:2:end-1);
values = v(2:2:end);
if any(~cellfun(@isstr,names)) || length(names)~=length(values)
    error('Options must be given by name-value pair.');
end
unknownOpts = ~ismember(names, optnames);
if any(unknownOpts)
    error('Unknown option(s): %s ', names{unknownOpts});
end
%Overwrite options by input
for i=1:length(names)
    p.(names{i}) = values{i};
end

end
