function h=plotwaves(X,srate,varargin)
%
% PLOTWAVES Plot multiple waveforms in a single panel
%

% Get constants and options
[nsig,len] = size(X);
if nargin<2 || isempty(srate)
    srate     = 1; %sampling in unit time
end
if ~isempty(varargin) && strcmp(varargin{1},'auto')
    scalerows = true; %row-wise scaling
    plotArgs = varargin(2:end);
else
    scalerows = false;
    plotArgs = varargin;
end
indices   = 1:nsig;

% Signal centering and rescaling
X = bsxfun(@minus, X, mean(X,2));
if scalerows
    X = spdiags(.5./max(abs(X),[],2),0,nsig,nsig)*X;
else
    X = .5.*X./max(abs(X(:)));
end
X=X.*2; %allow possible overlaps for visibility

% If srate is vector it directly specifies sampling points
switch length(srate)
  case 1
    timevec = (0:(len-1))./srate;
  case len
    timevec = srate;
  otherwise
    error(['Second argument must be a scalar or vector of the same length ' ...
           'as the input signals.']);
end

% Align the signals along Y-axis
X = X + repmat((nsig:-1:1)',1, len);

% Plot the signals from top to bottom
hdl = plot(timevec,X',plotArgs{:}); 
set(gca,'ytick',1:nsig, 'yticklabel', flipud(indices(:)));%nsig:-1:1
axis([timevec(1),timevec(end),.5,nsig+.5]);
%zoom xon;
%pan xon;

%-- if required, output the object handles
if nargout>0
    h = hdl;
end

return


